import 'dart:typed_data';
import 'package:convert/convert.dart';
import 'package:pointycastle/api.dart' show Digest;
import 'package:pointycastle/digests/ripemd160.dart';
import 'package:bs58check/bs58check.dart' as bs58check;

import '../configuration/network.dart';
import 'public_key.dart';

class Address {
  Uint8List bytes;
  
  static bool validate(String address, [int networkVersion]) {
    networkVersion ??= NetworkConfig.get().version();
    return bs58check.decode(address)[0] == networkVersion;
  }

  static Uint8List decode(String address) {
    return bs58check.decode(address);
  }

  Address(Uint8List bytes) {
    this.bytes = bytes;
  }

  Address.fromPublicKeyBuffer(Uint8List publicKey, [int networkVersion]) {
    Digest digest = new RIPEMD160Digest();
    digest.update(publicKey, 0, publicKey.length);
    Uint8List out = Uint8List(21);
    digest.doFinal(out, 1);

    networkVersion ??= NetworkConfig.get().version();

    out[0] = networkVersion;

    this.bytes = out;
  }

  factory Address.fromPublicKey(String publicKey, [int networkVersion]) {
    return Address.fromPublicKeyBuffer(hex.decode(publicKey), networkVersion);
  }

  factory Address.fromPassphrase(String passphrase, [int networkVersion]) {
    PublicKey pub = new PublicKey.fromPassphrase(passphrase);
    return Address.fromPublicKeyBuffer(pub.toBytes(), networkVersion);
  }

  @override
  String toString() {
    return bs58check.encode(this.bytes);
  }
}