import 'dart:convert';
import 'dart:typed_data';
import 'package:crypto/crypto.dart';
import 'package:convert/convert.dart';
import 'package:pointycastle/src/utils.dart';
import 'package:pointycastle/api.dart'
  show AsymmetricKeyPair;
import 'package:pointycastle/ecc/api.dart'
  show ECPrivateKey, ECPublicKey, ECDomainParameters;

class PrivateKey {
  ECPrivateKey privateKey;

  static final ECDomainParameters domain = new ECDomainParameters('secp256k1');

  PrivateKey.fromBytes(List<int> bytes) {
    BigInt d = decodeBigInt(bytes);
    this.privateKey = new ECPrivateKey(d, PrivateKey.domain);
  }

  factory PrivateKey.fromPassphrase(String passphrase) {
    Digest seed = sha256.convert(utf8.encode(passphrase));
    return PrivateKey.fromBytes(seed.bytes);
  }

  factory PrivateKey.fromHex(String privateKey) {
    List<int> seed = hex.decode(privateKey);
    return PrivateKey.fromBytes(seed);
  }

  ECPublicKey getPublicKey() {
    return ECPublicKey(PrivateKey.domain.G * this.privateKey.d, PrivateKey.domain);
  }

  AsymmetricKeyPair getKeyPair() {
    return new AsymmetricKeyPair(this.getPublicKey(), this.privateKey);
  }

  Uint8List toBytes() {
    return encodeBigInt(privateKey.d);
  }

  String toHex() {
    return hex.encode(this.toBytes());
  }
}