import '../configuration/fee.dart';
import '../identities/address.dart';
import '../identities/public_key.dart';
import '../enums/types.dart';
import 'transaction.dart';

class Builder {
  final Transaction _data = new Transaction();
  bool _signWithSenderAsRecipient = false;

  Builder.transfer(String recipientId, int amount) {
    this._data.type = Types.transfer;
    this._data.fee = FeeConfig.get(Types.transfer);
    this._data.recipientId = recipientId;
    this._data.amount = amount;
  }

  Builder.vote(String publicKey) {
    this._createVote(publicKey, false);
  }

  Builder.unvote(String publicKey) {
    this._createVote(publicKey, true);
  }

  Builder.delegateRegistration(String username) {
    this._data.type = Types.delegateRegistration;
    this._data.fee = FeeConfig.get(Types.delegateRegistration);
    this._data.asset = {
      'delegate': {
        'username': username
      }
    };
  }

  Builder.secondSignatureRegistration(String secondPassphrase) {
    this._data.type = Types.secondSignature;
    this._data.fee = FeeConfig.get(Types.secondSignature);
    this._data.asset = {
      'signature': {
        'publicKey': PublicKey.fromPassphrase(secondPassphrase).toHex()
      }
    };
  }

  Transaction get transaction {
    return this._data;
  }

  Builder network(int networkVersion) {
    this._data.network = networkVersion;
    return this;
  }

  Builder vendorField(String vendorField) {
    this._data.vendorField = vendorField;
    return this;
  }

  Builder fee(int fee) {
    this._data.fee = fee;
    return this;
  }

  Builder timestamp(int timestamp) {
    this._data.timestamp = timestamp;
    return this;
  }

  Transaction sign(String passphrase) {
    if (this._signWithSenderAsRecipient) {
      this._data.recipientId = Address.fromPassphrase(passphrase).toString();
    }
    this._data.sign(passphrase);
    this._data.id = this._data.getId();
    return this._data;
  }

  Transaction secondSign(String secondPassphrase) {
    return this._data.secondSign(secondPassphrase);
  }

  void _createVote(String publicKey, [bool isUnvote = false]) {
    this._signWithSenderAsRecipient = true;
    this._data.type = Types.vote;
    this._data.fee = FeeConfig.get(Types.transfer);
    StringBuffer vote = new StringBuffer();
    vote.write(isUnvote ? '-' : '+');
    vote.write(publicKey);
    this._data.asset = {
      'votes': [vote.toString()]
    };
  }
}
