import 'dart:convert';
import 'dart:typed_data';
import 'package:convert/convert.dart';

class Buffer {
  int _length = 0;
  Endian _order = Endian.little;
  List<Uint8List> _data = <Uint8List>[];

  int get length {
    return this._length;
  }

  void putHex(String input) {
    this.putList(hex.decode(input));
  }

  void putString(String input) {
    this.putList(utf8.encode(input));
  }

  void putInt(int byte) {
    this.putList([byte]);
  }

  void putInt32(int value) {
    ByteData bytes = new Uint8List(4).buffer.asByteData();
    bytes.setUint32(0, value, this._order);
    this.put(bytes.buffer.asUint8List());
  }

  void putInt64(int value) {
    ByteData bytes = new Uint8List(8).buffer.asByteData();
    bytes.setUint64(0, value, this._order);
    this.put(bytes.buffer.asUint8List());
  }

  void putList(List<int> bytes) {
    this.put(new Uint8List.fromList(bytes));
  }

  void put(Uint8List bytes) {
    this._data.add(bytes);
    this._length += bytes.length;
  }

  Uint8List toBytes() {
    Uint8List list = new Uint8List(this._length);
    int offset = 0;
    for (int i = 0; i < this._data.length; i++) {
      Uint8List item = this._data[i];
      list.setRange(offset, offset + item.length, item);
      offset += item.length;
    }
    return list;
  }
}

class BufferReader {
  Endian _order = Endian.little;
  int _offset = 0;
  ByteBuffer _buffer;

  BufferReader(Uint8List data) {
    this._buffer = data.buffer;
  }

  int get length {
    return this._buffer.lengthInBytes;
  }

  int get offset {
    return this._offset;
  }

  void skip(int length) {
    this._offset += length;
  }

  Uint8List get([int length]) {
    length ??= this.length - this._offset;
    final result = new Uint8List.view(this._buffer, this._offset, length);
    this._offset += length;
    return result;
  }

  String getHex([int length]) {
    return hex.encode(this.get(length));
  }

  int getInt() {
    final value = this._buffer.asByteData().getUint8(this._offset);
    this._offset += 1;
    return value;
  }

  int getInt32() {
    final value = this._buffer.asByteData().getUint32(this._offset, this._order);
    this._offset += 4;
    return value;
  }

  int getInt64() {
    final value = this._buffer.asByteData().getUint64(this._offset, this._order);
    this._offset += 8;
    return value;
  }
}